@@ -10,7 +10,7 @@
 #   IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 #   WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 #
-#   $Id: Embperl.pm,v 1.160 2001/06/05 04:56:20 richter Exp $
+#   $Id: Embperl.pm,v 1.176 2001/12/05 07:32:54 richter Exp $
 #
 ###################################################################################
 
@@ -64,7 +64,7 @@ use vars qw(
 @ISA = qw(Exporter DynaLoader);
 
 
-$VERSION = '1.3.3';
+$VERSION = '1.3.4';
 
 # HTML::Embperl cannot be bootstrapped in nonlazy mode except
 # under mod_perl, because its dependencies import symbols like ap_palloc
@@ -238,7 +238,7 @@ BEGIN
                         optRedirectStdout         optUndefToEmptyValue      optNoHiddenEmptyValue     optAllowZeroFilesize      
                         optKeepSrcInMemory        optKeepSpaces	       optOpenLogEarly           optNoUncloseWarn	       
                         } ;
-    @AliasHash   = qw{fdat udat mdat idat http_headers_out fsplitdat} ;
+    @AliasHash   = qw{fdat udat mdat sdat idat http_headers_out fsplitdat} ;
     @AliasArray  = qw{ffld} ;
     } ;
 
@@ -367,36 +367,38 @@ tie *OUT, 'HTML::Embperl::Out' ;
 use Text::ParseWords ;
 
 $SessionMgnt = 0 ;
+
+{
+my %sargs = (
+    lazy	   => 1,
+    create_unknown => 1,
+    ) ;
+my $session_handler = $ENV{EMBPERL_SESSION_HANDLER_CLASS} || 'Apache::SessionX' ; 
+my $ver = ''  ;
+
+if (defined ($ENV{EMBPERL_SESSION_ARGS}))
+    {
+    my @arglist = quotewords ('\s+', 0, $ENV{EMBPERL_SESSION_ARGS}) ;
+    foreach (@arglist)
+	{
+	/^(.*?)\s*=\s*(.*?)$/ ;
+	$sargs{$1} = $2 ;
+	}
+    }
+
 if (defined ($ENV{EMBPERL_SESSION_CLASSES}))
     { # Apache::Session 1.xx
     my ($os, $lm, $ser, $gen) = split /\s*,\s*|\s+/, $ENV{EMBPERL_SESSION_CLASSES} ;
     if (!$os || !$lm)
         {
         warn "[$$]SES:  EMBPERL_SESSION_CLASSES must be set properly (is $ENV{EMBPERL_SESSION_CLASSES})" ;
+        $session_handler = 'no' ;
         }
     else
         {
 	$ser ||= 'Storable' ;
 	$gen ||= 'MD5' ;
-        my @args  ;
-        if (defined ($ENV{EMBPERL_SESSION_ARGS}))
-            {
-	    my @arglist = quotewords ('\s+', 0, $ENV{EMBPERL_SESSION_ARGS}) ;
-	    foreach (@arglist)
-		{
-		/^(.*?)\s*=\s*(.*?)$/ ;
-		push @args, $1 ;
-		push @args, $2 ;
-		}
-            }
-
-        my %sargs = (
-	    @args,
-            lazy	   => 1,
-	    create_unknown => 1,
-	    ) ;
         
-        my $ver  ;
         if ($Apache::Session::VERSION =~ /^1\.0\d$/)
             {
 	    $sargs{object_store} = $os ;
@@ -413,31 +415,33 @@ if (defined ($ENV{EMBPERL_SESSION_CLASSES}))
             $ver = '>= 1.50' ;
 	    $DefaultIDLength = 32 ; 	
             }
-
-        my $session_handler = $ENV{EMBPERL_SESSION_HANDLER_CLASS} || 'HTML::Embperl::Session' ; 
-        eval "require $session_handler" ; 
-        die $@ if ($@)  ;
-
-	tie %mdat, $session_handler, undef, \%sargs ;
-	tie %udat, $session_handler, undef, {%sargs, recreate_id => 1} ;
-	$SessionMgnt = 2 ;
-	warn "[$$]SES:  Embperl Session management enabled ($ver)\n" if ($ENV{MOD_PERL}) ;
         }
     }
-elsif (exists $INC{'Apache/Session.pm'})
+
+if (defined ($ENV{EMBPERL_SESSION_CONFIG}))
     {
-    if ($Apache::Session::VERSION =~ /^0\.17/)
-        {
-        # Apache::Session = 0.17
-	##$SessionMgnt = 1 ;
-	##tie %udat, 'Apache::Session', $ENV{EMBPERL_SESSION_CLASS} || 'Win32',
-	##	      undef, {not_lazy=>0, autocommit=>0, lifetime=>&Apache::Session::LIFETIME} ;
-	##tie %mdat, 'Apache::Session', $ENV{EMBPERL_SESSION_CLASS} || 'Win32',
-	##	      undef, {not_lazy=>0, autocommit=>0, lifetime=>&Apache::Session::LIFETIME} ;
-	warn "[$$]SES:  Apache::Session 0.17 not supported by Embperl Session management anymore\n" ;
-	$SessionMgnt = 0 ;
+    $sargs{config} = $ENV{EMBPERL_SESSION_CONFIG} ;
+    }
+
+if ($session_handler ne 'no') 
+    { 
+    eval "require $session_handler" ;
+    if ($@)
+        { 
+        warn "[$$]SES:  Embperl Session management DISABLED beause of following error: $@\n" .
+             "Set \$ENV{EMBPERL_SESSION_HANDLER_CLASS} to 'no' before loading Embperl to avoid this message" if ($ENV{GATEWAY_INTERFACE}) ;
+        $SessionMgnt = 0 ;
+        }
+    else
+        {             
+        tie %mdat, $session_handler, undef, {%sargs, Transaction => 1} ;
+        tie %udat, $session_handler, undef, {%sargs, recreate_id => 1} ;
+        tie %sdat, $session_handler, undef, {%sargs, recreate_id => 1, newid => 1} ;
+        $SessionMgnt = 2 ;
+        warn "[$$]SES:  Embperl Session management enabled ($ver)\n" if ($ENV{MOD_PERL}) ;
         }
     }
+}
 
 
 #######################################################################################
@@ -616,6 +620,8 @@ sub CheckFile
         shift @path while ($skip--) ;
         my $fn = '' ;
         print LOG "[$$]Embperl path search Path: " . join (';',@path) . " Filename: $filename\n" if ($debug);
+        print LOG "[$$]Embperl path search pathskip = $pathskip  pathndx = $pathndx\n" if ($debug);
+
         #$$pathref = join (';', @path) ;
 
         foreach (@path)
@@ -678,7 +684,9 @@ sub ScanEnvironment
     $$req{'filesmatch'}  = $ENV{EMBPERL_FILESMATCH}  if (exists ($ENV{EMBPERL_FILESMATCH})) ;
     $$req{'decline'}     = $ENV{EMBPERL_DECLINE}     if (exists ($ENV{EMBPERL_DECLINE})) ;
     $$req{'debug'}       = $ENV{EMBPERL_DEBUG}   || 0 ;
+    $$req{'debug'} = oct($$req{'debug'}) if ($$req{'debug'} =~ /^0/) ; 
     $$req{'options'}     = $ENV{EMBPERL_OPTIONS} || 0 ;
+    $$req{'options'} = oct($$req{'options'}) if ($$req{'options'} =~ /^0/) ; 
     $$req{'log'}         = $ENV{EMBPERL_LOG}     || $DefaultLog ;
     $$req{'path'}        = $ENV{EMBPERL_PATH}    || '' ;
 
@@ -729,12 +737,6 @@ sub Execute
     elsif (exists $INC{'Apache.pm'})
         { $req_rec = Apache->request }
 
-    if (exists $$req{'debug'})
-	{ $$req{'debug'} = oct($$req{'debug'}) if ($$req{'debug'} =~ /^0/) ; }
-    if (exists $$req{'options'})
-	{ $$req{'options'} = oct($$req{'options'}) if ($$req{'options'} =~ /^0/) ; }
-
-
     if (defined ($$req{'virtlog'}) && $$req{'virtlog'} eq $$req{'uri'})
         {
         return SendLogFile ($DefaultLog, $ENV{QUERY_STRING}, $$req{'req_rec'}) ;
@@ -898,52 +900,54 @@ use strict ;
 	        %fdat = %{$$req{'fdat'}} ;
 	        @ffld = keys %fdat if (!defined ($$req{'ffld'})) ;
 	        }
-	    elsif (!($optDisableFormData) &&
+	    else
+                {
+                my $content_type = $req_rec?$req_rec -> header_in('Content-type'):$ENV{'CONTENT_TYPE'} ;
+                if (!defined ($import) && 
+                   !($optDisableFormData) &&
 	           !($r -> SubReq) &&
-	           defined($ENV{'CONTENT_TYPE'}) &&
-	           $ENV{'CONTENT_TYPE'}=~m|^multipart/form-data|)
-	        { # just let CGI.pm read the multipart form data, see cgi docu
-	        #eval 'require CGI' ;
-	        #die "require CGI failed: $@" if ($@); 
-	        require CGI ;
+	           $content_type &&
+	           ($content_type=~m|^multipart/form-data|))
+	            { # just let CGI.pm read the multipart form data, see cgi docu
+	            require CGI ;
 
-	        my $cgi ;
-	        #$cgi = new CGI  ;
-	        eval { $cgi = new CGI } ;
-	        if ($@ || !$cgi)
-                    {
-                    $r -> logerror (rcCGIError, $@)  ;
-                    $@ = '' ;
-                    }
-                else
-                    {
-	            @ffld = $cgi->param;
+	            my $cgi ;
+	            eval { $cgi = new CGI } ;
+	            if ($@ || !$cgi)
+                        {
+                        $r -> logerror (rcCGIError, $@)  ;
+                        $@ = '' ;
+                        }
+                    else
+                        {
+	                @ffld = $cgi->param;
     
-	            my $params ;
-    	            foreach ( @ffld )
-		        {
-    		        # the param_fetch needs CGI.pm 2.43
-		        #$params = $cgi->param_fetch( $_ ) ;
-    		        $params = $cgi->{$_} ;
-		        if ($#$params > 0)
+	                my $params ;
+    	                foreach ( @ffld )
 		            {
-		            $fdat{ $_ } = join ("\t", @$params) ;
-		            }
-		        else
-		            {
-		            $fdat{ $_ } = $params -> [0] ;
-		            }
-		        
-		        ##print LOG "[$$]FORM: $_=" . (ref ($fdat{$_})?ref ($fdat{$_}):$fdat{$_}) . "\n" if ($dbgForm) ; 
-		        print LOG "[$$]FORM: $_=$fdat{$_}\n" if ($dbgForm) ; 
+    		            # the param_fetch needs CGI.pm 2.43
+		            #$params = $cgi->param_fetch( $_ ) ;
+    		            $params = $cgi->{$_} ;
+		            if ($#$params > 0)
+		                {
+		                $fdat{ $_ } = join ("\t", @$params) ;
+		                }
+		            else
+		                {
+		                $fdat{ $_ } = $params -> [0] ;
+		                }
+		            
+		            ##print LOG "[$$]FORM: $_=" . (ref ($fdat{$_})?ref ($fdat{$_}):$fdat{$_}) . "\n" if ($dbgForm) ; 
+		            print LOG "[$$]FORM: $_=$fdat{$_}\n" if ($dbgForm) ; 
 
-		        if (ref($fdat{$_}) eq 'Fh') 
-		            {
-		            $fdat{"-$_"} = $cgi -> uploadInfo($fdat{$_}) ;
-		            }
-            	        }
-                    }
-	        }
+		            if (ref($fdat{$_}) eq 'Fh') 
+		                {
+		                $fdat{"-$_"} = $cgi -> uploadInfo($fdat{$_}) ;
+		                }
+            	            }
+                        }
+	            }
+                }
 
 	    my $saved_param = undef;
 	    if ( ref $$req{'param'} eq 'ARRAY') {
@@ -1100,6 +1104,7 @@ sub run (\@)
     my $cgi ;
     my $ioType ;
     my %req ;
+    my @param ;
 
     ScanEnvironment (\%req) ;
     
@@ -1117,6 +1122,11 @@ sub run (\@)
     	    shift @$args ;
     	    $req{'outputfile'} = shift @$args ;	
             }
+    	if ($$args[0] eq '-p')
+    	    {
+    	    shift @$args ;
+    	    push @param, shift @$args ;	
+            }
     	elsif ($$args[0] eq '-l')
     	    {
     	    shift @$args ;
@@ -1175,6 +1185,7 @@ sub run (\@)
     $req{'cleanup'} = 0 ;
     $req{'cleanup'} = -1 if (($req{'options'} & optDisableVarCleanup)) ;
     $req{'options'} |= optSendHttpHeader ;
+    $req{'param'} = \@param ;
 
     $rc = Execute (\%req) ;
 
@@ -1331,7 +1342,8 @@ sub cleanup
 		$varfile = GVFile (${*{"$package\::"}}{$key}) ;
 		#$varfile = GVFile (*ENTRY) ;
                 $glob = $package.'::'.$key ;
-		if (defined (*ENTRY{SCALAR}) && defined (${$glob}) && ref (${$glob}) eq 'DBIx::Recordset')
+		if (defined (*ENTRY{SCALAR}) && defined (${$glob}) && ref (${$glob}) eq 'DBIx::Recordset' &&
+                        !(defined ($addcleanup -> {$key}) && $addcleanup -> {$key} == 0))
 		    {
 		    print LOG "[$$]CUP:  Recordset $key\n" ;
 		    eval { DBIx::Recordset::Undef ($glob) ; } ;
@@ -1410,7 +1422,8 @@ sub cleanup
                 next if ($key =~ /^::/) ;
 	        local(*ENTRY) = $val;
 	        $glob = $package.'::'.$key ;
-		if (defined (*ENTRY{SCALAR}) && defined (${$glob}) && ref (${$glob}) eq 'DBIx::Recordset')
+		if (defined (*ENTRY{SCALAR}) && defined (${$glob}) && ref (${$glob}) eq 'DBIx::Recordset' &&
+                         !(defined ($addcleanup -> {$key}) && $addcleanup -> {$key} == 0))
 		    {
 		    eval { DBIx::Recordset::Undef ($glob) ; } ;
 		    print LOG "[$$]CUP:  Error: $@\n" if ($@) ;
@@ -1456,6 +1469,23 @@ sub cleanup
 
     @cleanups = () ;
 
+
+    if ($^O eq 'MSWin32' && $ENV{MOD_PERL})
+        {
+        # workaround for mod_perl problems with environment
+        foreach my $k (keys %ENV)
+            {
+            delete $ENV{$k} if ($k =~ /^EMBPERL/) ;
+            }
+        delete $ENV{'QUERY_STRING'} ;
+        delete $ENV{'CONTENT_LENGTH'} ;
+        delete $ENV{'CONTENT_TYPE'} ;
+        delete $ENV{'HTTP_COOKIE'} ;
+        }
+        
+
+
+
     flushlog () ;
 
     #log_svs ("cleanup exit") ;
@@ -1661,47 +1691,36 @@ sub SetupSession
     my $r ;
     $r = shift if (!(ref ($_[0]) =~ /^Apache/)) ;
     my ($req_rec, $Inputfile) = @_ ;
+    local $^W = 0 ;
 
     if ($HTML::Embperl::SessionMgnt && (!defined ($r) || !$r -> SubReq))
 	{
 	my $udat = tied(%HTML::Embperl::udat) ;
 	my $mdat = tied(%HTML::Embperl::mdat) ;
-	my $sessid ;
+	my $sdat = tied(%HTML::Embperl::sdat) ;
 	my $cookie_name = $r?$r -> CookieName:$ENV{EMBPERL_COOKIE_NAME} || 'EMBPERL_UID' ;
         my $cookie_val  = $ENV{HTTP_COOKIE} || ($req_rec?$req_rec->header_in('Cookie'):undef) ;
 
-	if (defined ($cookie_val) && ($cookie_val =~ /$cookie_name=(.*?)(\;|\s|$)/))
+	if ((defined ($cookie_val) && ($cookie_val =~ /$cookie_name=(.*?)(\;|\s|$)/)) || ($ENV{QUERY_STRING} =~ /$cookie_name=.*?:(.*?)(\;|\s|&|$)/) || $ENV{EMBPERL_UID} )
 	    {
-	    $sessid = $1 ;
-	    print HTML::Embperl::LOG "[$$]SES:  Received session cookie $1\n" if ($HTML::Embperl::dbgSession) ;
+	    print HTML::Embperl::LOG "[$$]SES:  Received user session id $1\n" if ($HTML::Embperl::dbgSession) ;
+	    $udat -> setid ($1) if (!$udat -> getid) ;
             }
 
-	if ($HTML::Embperl::SessionMgnt == 1)
-	    {
-            if (!$udat -> {ID})
-                {
-	        $udat -> {ID} = $sessid ;
-	        $udat -> {DIRTY} = 0 ;
-                }
+	$mdat -> setidfrom ($Inputfile) if ($Inputfile && !$mdat -> getid) ;
 
-            if ($Inputfile && !$mdat -> {ID})
-                {
-	        $mdat -> {ID} = substr(MD5 -> hexhash ($Inputfile), 0, &Apache::Session::ID_LENGTH );
-	        $mdat -> {DIRTY} = 0 ;
-                }
-	    }
-	else
+	if (($ENV{QUERY_STRING} =~ /${cookie_name}=(.*?)(\;|\s|&|:|$)/))
 	    {
-	    $udat -> setid ($sessid) if (!$udat -> getid) ;
-	    $mdat -> setid (substr(MD5 -> hexhash ($Inputfile), 0, $mdat -> {args} -> {IDLength} || $HTML::Embperl::DefaultIDLength)) if ($Inputfile && !$mdat -> getid)
-	    }
+	    print HTML::Embperl::LOG "[$$]SES:  Received state session id $1\n" if ($HTML::Embperl::dbgSession) ;
+	    $sdat -> setid ($1) if (!$sdat -> getid) ;
+            }
 	}
     else
         {
         return undef ; # No session Management
         }
 
-    return wantarray?(\%HTML::Embperl::udat, \%HTML::Embperl::mdat):\%HTML::Embperl::udat ;
+    return wantarray?(\%HTML::Embperl::udat, \%HTML::Embperl::mdat, \%HTML::Embperl::sdat):\%HTML::Embperl::udat ;
     }
 
 #######################################################################################
@@ -1713,14 +1732,7 @@ sub GetSession
 	{
 	my $udat = tied(%HTML::Embperl::udat) ;
 
-	if ($HTML::Embperl::SessionMgnt == 1)
-	    {
-            return wantarray?(\%HTML::Embperl::udat, \%HTML::Embperl::mdat):\%HTML::Embperl::udat ;
-	    }
-	else
-	    {
-	    return wantarray?(\%HTML::Embperl::udat, \%HTML::Embperl::mdat):\%HTML::Embperl::udat  ;
-	    }
+        return wantarray?(\%HTML::Embperl::udat, \%HTML::Embperl::mdat, \%HTML::Embperl::sdat):\%HTML::Embperl::udat ;
 	}
     else
         {
@@ -1772,35 +1784,11 @@ sub CleanupSession
 	{
 	my $udat = tied(%HTML::Embperl::udat) ;
 	my $mdat = tied(%HTML::Embperl::mdat) ;
+	my $sdat = tied(%HTML::Embperl::sdat) ;
 
-	if ($HTML::Embperl::SessionMgnt & 1)
-	    {
-	    if ($udat->{'DIRTY'})
-		{
-		print HTML::Embperl::LOG "[$$]SES:  Store session data of \%HTML::Embperl::udat id=$udat->{ID}\n" if ($HTML::Embperl::dbgSession) ;
-		$udat->{'DATA'}->store ;
-		}
-	    else
-		{
-		print HTML::Embperl::LOG "[$$]SES:  session data not dirty, do not store \%HTML::Embperl::udat\n" if ($HTML::Embperl::dbgSession) ;
-		}
-
-	    $udat->{'DATA'} = undef ;
-	    $udat -> {ID}	= undef ;
-	    if ($mdat->{'DIRTY'})
-		{
-		print HTML::Embperl::LOG "[$$]SES:  Store session data of \%HTML::Embperl::mdat id=$mdat->{ID}\n" if ($HTML::Embperl::dbgSession) ;
-		$mdat->{'DATA'}->store ;
-		}
-
-	    $mdat->{'DATA'} = undef ;
-	    $mdat -> {ID}	= undef ;
-	    }
-	else
-	    {
-	    $udat -> cleanup ;
-	    $mdat -> cleanup ;
-	    }
+	$udat -> cleanup ;
+	$mdat -> cleanup ;
+	$sdat -> cleanup ;
 	}
     }
 
