@@ -144,7 +144,7 @@ extern "C" {
 #endif
 
 
-module MODULE_VAR_EXPORT embperl_module;
+void embperl_ApacheAddModule () ;
 
 
 #endif
@@ -220,7 +220,8 @@ tFile * SetupFileData   (/*i/o*/ register req * r,
 
 tFile * GetFileData     (/*in*/  char *  sSourcefile,
                          /*in*/  char *  sPackage,
-                         /*in*/  double  mtime) ;
+                         /*in*/  double  mtime,
+			 /*in*/  int     bEP1Compat) ;
 
 			 
 tConf * SetupConfData   (/*in*/ HV *   pReqInfo,
@@ -438,6 +439,12 @@ void OutputEscape (/*i/o*/ register req * r,
  		   /*in*/  int            nDataLen,
  		   /*in*/  struct tCharTrans *   pEscTab,
  		   /*in*/  char           cEscChar) ;
+SV * Escape	  (/*i/o*/ register req * r,
+ 		   /*in*/  const char *   sData,
+ 		   /*in*/  int            nDataLen,
+ 		   /*in*/  int            nEscMode,
+ 		   /*in*/  struct tCharTrans *   pEscTab,
+ 		   /*in*/  char           cEscChar) ;
 
 int TransHtml (/*i/o*/ register req * r,
 		/*i/o*/ char *         sData,
