@@ -10,7 +10,7 @@
 #   IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
 #   WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 #
-#   $Id: epnames.h,v 1.28 2001/05/29 06:28:11 richter Exp $
+#   $Id: epnames.h,v 1.30 2001/12/04 07:43:15 richter Exp $
 #
 ###################################################################################*/
 
@@ -194,7 +194,8 @@
 #define xDomTreeAttr		    EMBPERL_xDomTreeAttr		   
 #define xNoName			    EMBPERL_xNoName			   
 #define xOrderIndexAttr		    EMBPERL_xOrderIndexAttr		   
- 
+#define Escape		            EMBPERL_Escape		   
+#define GetSessionID		    EMBPERL_GetSessionID 
  
 #ifndef PERL_VERSION
 #include <patchlevel.h>
@@ -392,4 +393,5 @@
 
 #endif /* endif WIN32 */
 
-#endif /* APACHE */
\ No newline at end of file
+#endif /* APACHE */
+
